@@ -31,14 +31,17 @@ sub truncfd ($)
 	my ($dummy1, $bytepersect, $sectperclus, $resvsectors, $fats, $rootdirents,
 		$sectors, $dummy2, $sectperfat, $sectpertrk, $heads, $hidsectors,
 		$bigsectors, $dummy3, $fstype) =
-		unpack('a11vCvCvvavvvVVa19a5', $buffer);
+		unpack('a11vCvCvvavvvVVa18a5', $buffer);
 	if ($sectors == 0) {
 	    $sectors = $bigsectors;
 	}
 	$cylinders = $sectors / ($sectpertrk * $heads);
-	$clusters = $sectors / $sectperclus;
 	$fatsectors = $fats * $sectperfat;
 	$rootdirsectors = POSIX::ceil(($rootdirents * 32) / $bytepersect);
+	$clusstart  = $resvsectors + $fatsectors + $rootdirsectors;
+	$clusters = int (($sectors - $clusstart) / $sectperclus);
+	return (-s $file) unless $clusters < 65525;
+	$fstype = $clusters < 4085? 'FAT12':'FAT16' if ($fstype ne 'FAT12' && $fstype ne 'FAT16');
 	if (DEBUG) {
 		print STDERR <<EOF;
 $fstype filesystem:
@@ -80,12 +83,11 @@ EOF
 	$buffer = pack("L*", @fatents);
 	# Then extract in little endian format
 	@fatents = unpack("v*", $buffer);
-	$#fatents = $clusters if ($clusters < $#fatents);
+	$#fatents = $clusters + 1 if (($clusters + 1) < $#fatents);
 	foreach $i (reverse(0..$#fatents)) {
 		$lastclus = $i, last if ($fatents[$i] != 0);
 	}
 	print STDERR "Last used cluster is $lastclus\n" if (DEBUG);
-	$clusstart = $resvsectors + $fatsectors + $rootdirsectors;
 	$lastsector = $clusstart + ($lastclus - 1) * $sectperclus;
 	print STDERR "Last used sector is $lastsector\n" if (DEBUG);
 	return ($lastsector * 512);
