@@ -1,12 +1,9 @@
 #include	"etherboot.h"
+#include	"start32.h"
 
 /* Taken from Etherboot */
 /* by Eric Biederman */
 
-extern unsigned int memsize P((void));
-extern unsigned short basememsize P((void));
-extern int meme820(struct e820entry *buf, int count);
-
 struct meminfo meminfo;
 
 void get_memsizes(void)
