@@ -26,7 +26,7 @@ use constant LUA_VERSION => 0x04000100;	# 4.0.1.0
 use bytes;
 
 use vars qw($libdir $version $format $target $output $module $relocseg $relocsegstr
-	$progreturns $param $append $rootdir $rootmode $ip $ramdisk $rdbase
+	$progreturns $param $append $rootdir $rootmode $ip $ramdisk $rdbase $rdnopad
 	$simhd $dishd $squashfd $first32 $showversion);
 
 sub check_file
@@ -223,7 +223,7 @@ sub mknbi_linux ($)
 		end => 1 };
 	$ramdiskseg = { file => $ramdisk,
 		segment => 0x10000,
-		align => 4096,
+		align => $rdnopad ? 1 : 4096,
 		id => 21,
 		end => 1 };
 	$$kernelseg{'end'} = 0 if (defined($ramdisk));
@@ -545,6 +545,7 @@ $libdir = '@@LIBDIR@@';		# where config and auxiliary files are stored
 
 $version = '@@VERSION@@';
 $showversion = '';
+$rdnopad = 0;
 $simhd = 0;
 $dishd = 0;
 $squashfd = 1;
@@ -559,6 +560,7 @@ GetOptions('format=s' => \$format,
 	'rootmode=s' => \$rootmode,
 	'ip=s' => \$ip,
 	'rdbase=s' => \$rdbase,
+	'rdnopad!' => \$rdnopad,
 	'harddisk!' => \$simhd,
 	'disableharddisk!' => \$dishd,
 	'squash!' => \$squashfd,
@@ -572,7 +574,7 @@ if ($showversion) {
 	exit 0;
 }
 
-if (defined($ENV{LANG}) and $ENV{LANG} =~ /\.UTF-8$/i) {
+if ($] > 5.008 and $] < 5.008003 and defined($ENV{LANG}) and $ENV{LANG} =~ /\.UTF-8$/i) {
 	print STDERR <<'EOF';
 Warning: Perl 5.8 may have a bug that affects handing of strings in Unicode
 locales that may cause misbehaviour with binary files.  To work around this
@@ -596,7 +598,7 @@ binmode(STDOUT);
 
 if ($format eq 'nbi') {
 	$first32 = '' if !defined($first32);
-	if ($target ne 'rom' and $target ne 'dos' and $target ne 'fdos') {
+	if ($target ne 'rom') {
 		print STDERR "mkelf-$target is preferred in future instead of mknbi-$target\n";
 	}
 	$module = Nbi->new($libdir);
@@ -796,9 +798,12 @@ kernel in memory, if the kernel is loaded high. For this option to work,
 the kernel must be able to handle ramdisks at these addresses.
 I<0xNNNNNNNN> moves the ramdisk to the hex address specified. The onus
 is on the user to specify a suitable address that is acceptable to the
-kernel and doesn't overlap with any other segments. It will have to be
-aligned to a 4k byte boundary so you should ensure that this is so. (The
-last three hex digits must be 0.)
+kernel and doesn't overlap with any other segments. Etherboot will round
+address down to multiple of 4k (last 3 digits to zero).
+
+B<--rdnopad> By default, etherboot pads (with nulls) the given initrd of
+any size to multiple of 4k bytes (aligning to memory page boundary). Use
+this option to disable padding if it causes problem.
 
 B<--first32=>I<program> Override the default first stage setup
 program.  It can be used to call extensions to the Etherboot code, which
