@@ -57,3 +57,17 @@ char *strncat(char *dest, const char *src, unsigned int maxlen)
 	*d = 0;
 	return (dest);
 }
+
+#ifndef __HAVE_ARCH_MEMCMP
+int memcmp(const void *s1, const void *s2, int n)
+{
+	int i;
+	const unsigned char *src1 = s1, *src2 = s2;
+	for(i = 0; i < n ; i++) {
+		if (src1[i] != src2[i]) {
+			return src1[i] - src2[i];
+		}
+	}
+	return 0;
+}
+#endif
